mod client;
mod config;
mod roll_bot;

use structopt::StructOpt;

use log::{error, info};
use std::error::Error;
use std::time::Duration;

use crate::client::Client;
use crate::config::Args;
use crate::roll_bot::RollBot;
use rppal::pwm::{Polarity, Pwm};

fn main() -> Result<(), Box<dyn Error>> {
    env_logger::init();

    let args = Args::from_args();

    let config = config::RollBotConfig::new(args.config_file);

    info!("Initializing PWM");
    let pwm = Pwm::with_period(
        config.pwm_config.pwm_channel.clone().into(),
        Duration::from_millis(config.pwm_config.period_ms),
        Duration::from_micros(config.pwm_config.pulse_max_us),
        Polarity::Normal,
        true,
    )?;

    info!("Initializing Roll Bot");
    let roll_bot = RollBot {
        pwm,
        steps: config.pwm_config.steps.clone(),
    };

    roll_bot.set_pwm_output(Duration::from_micros(config.pwm_config.pulse_min_us))?;

    if args.test {
        let res = roll_bot.roll(3);

        match res {
            Ok(_) => info!("Test completed, exiting..."),
            Err(err) => error!("Error rolling die: {}", err),
        }
    } else {
        let mut client = Client::new(config, roll_bot);
        let res = client.run_client();

        if let Err(err) = res {
            error!("Error in client: {}", err);
        }

        info!("Roll Bot shutting down...");
    }

    Ok(())
}
